function [dc] = GMMMPEC_dc(x0)

% GMMMPEC_dc
% Gradient of the constraints of the random coefficients Logit estimated
% via MPEC.
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global x IV K prods v nn T
global cholesky setuprc

theta1 = x0(1:K+1, 1);
if cholesky==K,
    theta2 = x0(K+2:2*K+2, 1);
    delta = x0(2*K+3:2*K+2+T*prods, 1);
    g = x0(2*K+3+T*prods:end, 1);
elseif cholesky==K*(K-1),
    theta2 = x0(K+2:K+1+length(setuprc),1);
    delta = x0(K+1+length(setuprc)+1:K+1+length(setuprc)+T*prods, 1);
    g = x0(K+1+length(setuprc)+T*prods+1:end, 1);
end

MU = mufuncnormMPEC(theta2);
expmu = exp(MU); 
expmval = exp(delta);
[EstShare, simShare] = ind_shnormMPEC(expmval,expmu);

size(EstShare);

size(simShare);

nx0 = size(x0,1);
ng = size(g,1);

ooo = ones(1,K+1);
ooo1 = ones(prods,1);
dSdtheta2 = zeros(T*prods,K+1);
dSddeltaDIAG = zeros(T*prods,prods);
dSddelta = zeros(T*prods, T*prods);

for tt=1:T,
    index = (0:prods-1)'*T+tt;
    for rr = 1:nn,
        dSddeltaDIAG(index,:) = dSddeltaDIAG(index,:) + (diag(simShare(index,rr)) - simShare(index,rr)*simShare(index,rr)')/nn;
        dSdtheta2(index,:) = dSdtheta2(index,:) + (simShare(index,rr)*ooo).*(ooo1*v(:,rr)').*( x(index,:) - (ooo1*(simShare(index,rr)'*x(index,:))))/nn;
    end
    dSdtheta2(index,:) = dSdtheta2(index,:) ./ (EstShare(index)*ooo);
    dSddeltaDIAG(index,:) = dSddeltaDIAG(index,:) ./ (EstShare(index)*ooo1');
end

dSddelta = zeros(T*prods, T*prods);

for i = 1:prods
    for j = 1:prods
    dSddelta( (j-1)*T+1:j*T, (i-1)*T+1:i*T ) = diag(dSddeltaDIAG((j-1)*T+1:j*T, i)); 
    end
end

dc1 = [zeros(T*prods,K+1), dSdtheta2, dSddelta, zeros(T*prods,ng)];
dc2 = [IV'*x, zeros(ng, K+1), -IV', eye(ng)];
dc = [dc1; dc2];
