function [grad_f] = GMMMPEC_grad(x0, Prob)

% GMMMPEC_grad
% Gradient of the GMM objective function for the random coefficients Logit
% estimated via MPEC.
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global W K prods T
global cholesky setuprc

theta1 = x0(1:K+1, 1);
if cholesky==K,
    theta2 = x0(K+2:2*K+2, 1);
    delta = x0(2*K+3:2*K+2+T*prods, 1);
    g = x0(2*K+3+T*prods:end, 1);
    nx0 = size(x0,1);
    grad_f = zeros(nx0,1);
    grad_f(2*K+3+T*prods:end,1) = 2*W*g;
elseif cholesky==K*(K-1),
    theta2 = x0(K+2:K+1+length(setuprc),1);
    delta = x0(K+1+length(setuprc)+1:K+1+length(setuprc)+T*prods, 1);
    g = x0(K+1+length(setuprc)+T*prods+1:end, 1);
    nx0 = size(x0,1);
    grad_f = zeros(nx0,1);
    grad_f(K+1+length(setuprc)+T*prods+1:end,1) = 2*W*g;
end
