global nn K v report covX costparams betatrue rc rctrue oo report nopurchglobal

%%%%%%%%%%%%%%%%%%%%%%%
% Monte Carlo Simulations to compare BLP/NFP versus MPEC
%
% source: Dube, Fox and Su (2008)
% Code Revised: May 2008
%%%%%%%%%%%%%%%%%%%%%%%

diary test.out;

randn('seed',155)                                   % reset normal random number generator
rand('seed',155)                                    % reset uniform random number generator
nn = 100;                                           % # draws to simulate shares
oo = ones(1,nn);                                    % for use in simulation of shares

costparams = ones(6,1)/2;
betatrue = [.1 1.5 1.5 .5 -3]';                     % true mean tastes
K = size(betatrue,1)-1;                             % # attributes
covrc = diag( [ .5 .5 .5 .5 .2] );                  % true co-variances in tastes
rctrue = covrc(find(covrc));
thetatrue = [betatrue;rctrue];
v = randn(length(betatrue),nn);                     % draws for share integrals during estimation
rc = chol(covrc)'*v;                                % draws for share integrals for data creation
sigmaxi=1;
covX = -.8*ones(K-1)+1.8*eye(K-1);                  % allow for correlation in X's
covX(1,3) = .3;
covX(2,3) = .3;
covX(3,1) = .3;
covX(3,2) = .3;

report =100;

% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%% Vary the Starting Values                        %%
% %%%  - check how often we get correct solution      %% 
% %%%  - only use first 5 simulated datasets          %%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
betatrue = [-1 1.5 1.5 .5 -3]';                     % re-set true mean tastes

prods = 25;
T = 50;
weak = 4;                                           % weakness of the instruments
sims = 1;                                           % # replications per experiment
BLP=1;
MPEC=1;
elasdo=1;
outputtable = [];
rand('seed',50000);                                 % re-set uniform random number generator
reps = 100;
theta0 = rand(K+1,reps)*7;
for jj = 1:reps,
    display(['Starting case with starting value # ' num2str(jj)])
    rand('seed',200);
    randn('seed',200);
    [theta,Lipschitz] = MC(prods,T,sims,weak,sigmaxi,BLP,MPEC,elasdo,theta0(:,jj),0);       % supply the starting value
    Lipschtrue = Lipschitz(:,1);
    if BLP==1,LipschBLP = Lipschitz(:,2);end;
    load 2sls
    load Elasbartrue    
    save(['MCoutBLP_' num2str(T) 'mkts_' num2str(prods) 'products_' 'startingvalue' num2str(jj)],'betaols','beta2sls','seols','se2sls','betatrue','Elasbartrue', 'Lipschitz','nopurchglobal');
    if BLP==1,
        load GMMstep1tight
        load ElasbarBLP
        biastight = mean( repmat(thetatrue,1,sims)-thetaGMM1tight ,2);
        RMSEtight = sqrt(mean( (repmat(thetatrue,1,sims)-thetaGMM1tight).^2 ,2));
        numcorrecttight = sum( INFOGMM1tight==0);
        save(['MCoutNFPtight_' num2str(T) 'mkts_' num2str(prods) 'products' 'startingvalue' num2str(jj)],'thetaGMM1tight', 'W', 'Jstat1tight', 'timeGMM1tight', 'FuncEvGMM1tight', 'GradEvGMM1tight', 'covGMM1tight', 'repsGMM1tight', 'INFOGMM1tight', 'ElasbarBLP', 'LipschBLP','innerlooptimetight','biastight','RMSEtight','numcorrecttight');        
        
        load GMMstep1loose
        load ElasbarBLPloose
        biasloose = mean( repmat(thetatrue,1,sims)-thetaGMM1loose ,2);
        RMSEloose = sqrt(mean( (repmat(thetatrue,1,sims)-thetaGMM1loose).^2 ,2));
        numcorrectloose = sum( INFOGMM1loose==0);
        save(['MCoutNFPloose_' num2str(T) 'mkts_' num2str(prods) 'products_'  'startingvalue' num2str(jj)],'thetaGMM1loose', 'W', 'Jstat1loose', 'timeGMM1loose', 'FuncEvGMM1loose', 'GradEvGMM1loose', 'covGMM1loose', 'repsGMM1loose', 'INFOGMM1loose', 'ElasbarBLPloose','LipschBLP','innerlooptimeloose','biasloose','RMSEloose','numcorrectloose');

        load GMMstep1loose1
        load ElasbarBLPloose1
        biasloose1 = mean( repmat(thetatrue,1,sims)-thetaGMM1loose1 ,2);
        RMSEloose1 = sqrt(mean( (repmat(thetatrue,1,sims)-thetaGMM1loose1).^2 ,2));
        numcorrectloose1 = sum( INFOGMM1loose1==0);
        save(['MCoutNFPloose1_' num2str(T) 'mkts_' num2str(prods) 'products_' 'startingvalue' num2str(jj)],'thetaGMM1loose1', 'W', 'Jstat1loose1', 'timeGMM1loose1', 'FuncEvGMM1loose1', 'GradEvGMM1loose1', 'covGMM1loose1', 'repsGMM1loose1', 'INFOGMM1loose1', 'ElasbarBLPloose1','LipschBLP','innerlooptimeloose1','biasloose1','RMSEloose1','numcorrectloose1');
    end
    if MPEC==1,
        load GMMMPEC
        load ElasbarMPEC
        biasMPEC = mean( repmat(thetatrue,1,sims)-thetaGMMMPEC ,2);
        RMSEMPEC = sqrt(mean( (repmat(thetatrue,1,sims)-thetaGMMMPEC).^2 ,2));
        numcorrectMPEC = sum( INFOGMMMPEC==0);
        save(['MCoutMPEC_' num2str(T) 'mkts_' num2str(prods) 'products_' 'startingvalue' num2str(jj)],'thetaGMMMPEC', 'W', 'JstatGMMMPEC', 'timeGMMMPEC', 'FuncEvGMMMPEC', 'GradEvGMMMPEC', 'covGMMMPEC', 'INFOGMMMPEC', 'ElasbarMPEC','biasMPEC','RMSEMPEC','numcorrectMPEC');
    end
    
    rmseelasJP
    out =       [ [numcorrectloose;numcorrectloose1;numcorrecttight;numcorrectMPEC] , [timeloosebar;timeloose1bar;timetightbar;timeMPECbar] , [ownelasbarGMMlooseBIAS;ownelasbarGMMloose1BIAS;ownelasbarGMMtightBIAS;ownelasbarGMMMPECBIAS] , [ownelasbarGMMlooseRMSE;ownelasbarGMMloose1RMSE;ownelasbarGMMtightRMSE;ownelasbarGMMMPECRMSE]];
    outconv =   [[timeloosebarconv;timeloose1barconv;timetightbarconv;timeMPECbarconv] , [ownelasbarGMMlooseBIASconv;ownelasbarGMMloose1BIASconv;ownelasbarGMMtightBIASconv;ownelasbarGMMMPECBIASconv] , [ownelasbarGMMlooseRMSEconv;ownelasbarGMMloose1RMSEconv;ownelasbarGMMtightRMSEconv;ownelasbarGMMMPECRMSEconv]];
    outnoconv = [[timeloosebarnoconv;timeloose1barnoconv;timetightbarnoconv;timeMPECbarnoconv] , [ownelasbarGMMlooseBIASnoconv;ownelasbarGMMloose1BIASnoconv;ownelasbarGMMtightBIASnoconv;ownelasbarGMMMPECBIASnoconv] , [ownelasbarGMMlooseRMSEnoconv;ownelasbarGMMloose1RMSEnoconv;ownelasbarGMMtightRMSEnoconv;ownelasbarGMMMPECRMSEnoconv]];
    save(['tables_' num2str(T) 'mkts_' num2str(prods) 'products_' 'startingvalue' num2str(jj)],'out','outconv','outnoconv');
    outputtable = [ outputtable ; [out,outconv,outnoconv] ];
end


