function Lipschitz = mklipschitz(expmval,expmu)

% mklipschitz
%
% L = mklipschitz(expmval,expmu) computes the Lipschitz contant for the BLP
% contraction-mapping taking the exponentiated mean utilities, expmval, and "nn" draws of exponentiated
% deviations from mean utilities, expmu.  This can be used to measure the
% exact rate of convergence for the inner loop in the NFP estimator proposed in BLP.
% see Dube, Fox and Su (2008) for the derivation.
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global T prods nn K
global oo sharesum denomexpand

mval = (expmval*oo ).*expmu;           % this is the numerator (oo speeds-up expanding mean utility by number of draws)
sum1 = sharesum*mval;
sum11 = 1./(1+sum1);                   % this is the denominator of the shares
denom1 = sum11(denomexpand,:);         % this expands the denominator
S = mval.*denom1;       % simulated shares for each draw
sh = mean(S,2);                        % expected share (i.e. mean across draws)

ooo = ones(1,K+1);
ooo1 = ones(prods,1);

dSddeltaDIAG = zeros(T*prods,prods);
dSddelta = zeros(T*prods, T*prods);

% L = zeros(T*prods,prods);
% for tt = 1:T,
%     for ii = 1:nn,
%         index = (0:prods-1)*T + tt;
%         scale = 1./repmat(sh(index)',prods,1);
%         SS = S(index,ii);
%         L((tt-1)*prods+1:tt*prods,:) = L((tt-1)*prods+1:tt*prods,:) + (diag(SS) - SS*SS').*scale/nn;
%     end
% end

for tt=1:T,
    index = (0:prods-1)'*T+tt;
    for ii = 1:nn,
        SS = S(index,ii);
        dSddeltaDIAG(index,:) = dSddeltaDIAG(index,:) + (diag(SS) - SS*SS')/nn;
    end
    
    dSddeltaDIAG(index,:) = dSddeltaDIAG(index,:) ./ (sh(index)*ooo1');
end

dSddelta = zeros(T*prods, T*prods);

for i = 1:prods
    for j = 1:prods
    dSddelta( (j-1)*T+1:j*T, (i-1)*T+1:i*T ) = diag(dSddeltaDIAG((j-1)*T+1:j*T, i));  
    end
end

L = eye(T*prods) - dSddelta;
Lipschitz = norm(L,inf);

