function [share,nopurch] = mksharesim(betatrue,M,T,x,xi,oo,oo1,rc)

%%%%%%%%%%%%
% MKSHARESIM
% generates market share data.
% If number of draws is large (i.e. larger than 15,000)
% then use mksharesimBIG.m
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global euler prods datasort

delta = x*betatrue+xi;

if isinf(M)==1,             % Continuum of consumers
    if nargin==8,
        MU = x*rc;
        numer = exp( repmat(delta,1,size(rc,2)) + MU );
    else,
        numer = exp(delta);
    end
    temp = cumsum(numer(datasort,:));
    temp1 = temp( (1:T)*prods,: );
    temp1(2:end,:) = diff(temp1);
    denom = 1+temp1;
    share = mean(numer./repmat(denom,prods,1),2);
    nopurch = mean(1./denom,2);
else,                       % Finite Number of consumers
    U = reshape( -(evrnd( oo,1)+euler) , M*T, prods+1);
    U(:,1:prods) = U(:,1:prods) + oo1*reshape( delta, T, prods);
    [umax,choice] = max( U,[],2 );
    
    Q = zeros(T,prods);
    nopurch = zeros(T,1);
    for tt = 1:T,
        for jj = 1:prods,
            Q(tt,jj) = sum( choice( (tt-1)*M+1:tt*M )==jj);
        end
        nopurch(tt) = sum( choice( (tt-1)*M+1:tt*M )==prods+1);
    end
    share = reshape( Q/M , T*prods , 1);
    nopurch = nopurch/M;
end
