function [theta,G,INFO,CPUt,FuncEval,GradEval,reps,Dg,cov] = runGMMBLPTomlab(theta00,W,mvalold0,deriv,gradient)

% run GMMBLPTomlab
% Run the GMM estimation of the random coefficients Logit demand model estimated via NFP.
%
% source: Dube, Fox and Su (2008)
% Code Revised: May 2008

global iteration mvalold theta1 T prods tol_inner tol_outer rr

if nargin<5, gradient=0; end;

iteration = 0;
INFO = -1;
reps =0;
CPUt = 0;
mvalold = mvalold0;
differ = 1;
while INFO == -1 && reps <= 1,
    % Setup the lower and upper bounds for varialbes and constraints

    [thetalow,thetaupp,Flow,Fupp] = GMMBLPSetup(theta00);
    
    if tol_inner==1.e-14,
        Name = 'BLP NFP Problem with tight inner-loop tolerance';
    else
        Name = 'BLP NFP Problem with loose inner-loop tolerance';
    end
    
    fLowBnd = 0;         % Lower bound on function.
    if gradient==1,
        Prob = conAssign('gmmobjnormsnopt', 'analyticderiv', [], [], thetalow, thetaupp, Name, theta00,...
            [], fLowBnd, [], [], [], [], [], [], [], [], []);    
    else
        Prob = conAssign('gmmobjnormsnopt', [], [], [], thetalow, thetaupp, Name, theta00,...
            [], fLowBnd, [], [], [], [], [], [], [], [], []);    
        Prob.ConsDiff = 6;
        Prob.NumDiff = 6;
    end
    %Prob.PriLevOpt = 1; 
    %Prob.PriLevOpt = 3; 
    %Prob.KNITRO.PrintFile = ''; 
    Prob.KNITRO.options.ALG = 1;
    Prob.KNITRO.options.MAXIT = 100;    % Setting maximum number of iterations 
    Prob.KNITRO.options.HESSOPT = 2;    % to be used along with first-order derivatives
    
    Prob.KNITRO.options.OPTTOL = tol_outer ;    % set outer-loop tolerance
    
    BLP_Result = tomRun('knitro', Prob, 1);

    theta = BLP_Result.x_k;
    G = BLP_Result.f_k;
    INFO = BLP_Result.Inform;
    CPUt = BLP_Result.CPUtime + CPUt;
    
    FuncEval = BLP_Result.FuncEv;
    GradEval = BLP_Result.GradEv; 
    
%    diff = max(abs(theta00-theta)./theta);
    theta00 = theta;
    reps = reps+1;
end

if deriv==1,    %4
    Dg = mkderivGMMnorm(theta1,theta,1.e-5);
    cov = inv(Dg*W*Dg')/(T*prods);
end

disp(['Monte Carlo Replication ' num2str(rr)])
disp(['# runs of optimizer     ' num2str(reps)])
disp(' ')
