function [theta1,theta2,G,INFO,CPUt,FuncEval,GradEval,Dg,cov] = runGMMMPECTomlab(x00,deriv,gradient)

% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global iteration theta1 W K T prods IV tol_outer rr

iteration = 0;
INFO = -1;
reps =0;
CPUt = 0;
differ = 1;
while INFO == -1 && reps <= 1,
    
    % Setup the lower and upper bounds for varialbes and constraints
    [xlow,xupp,Flow,Fupp] = GMMMPECSetup3(x00);
    
    x_L = xlow ;   %% % Lower bounds for x.
    x_U = xupp ;   %% % Upper bounds for x.
    
    c_L = Flow(2:end);  % Lower bound on nonlinear constraints.
    c_U = Fupp(2:end);  % Upper bound on nonlinear constraints. 
    
    c = GMMMPEC_c(x00);
    f = GMMMPEC_f(x00);
    
    nx0 = size(x00,1);
    HessPattern = zeros(nx0,nx0);
    HessPattern(2*K+3+T*prods:end,2*K+3+T*prods:end) = 1;
    
    % dc_x0 = GMMMPEC_dc(x0); 
    
    nIV = size(IV,2);
    
    c11 = zeros(T*prods, K+1);
    c12 = ones(T*prods, K+1);
    c13 = repmat(eye(T), prods, prods);
    c14 = zeros(T*prods, nIV);
    
    c21 = ones(nIV, K+1);    
    c22 = zeros(nIV, K+1);    
    c23 = ones(nIV, T*prods);
    c24 = eye(nIV);     
    
    ConsPattern = [ c11 c12 c13 c14; c21 c22 c23 c24 ];
    
    Name = 'BLP MPEC Problem';
    fLowBnd = 0;         % Lower bound on function.
    
    if nargin==3,
	    Prob = conAssign('GMMMPEC_f', 'GMMMPEC_grad', 'GMMMPEC_hess', HessPattern, x_L, x_U, Name, x00,... 
        	[], fLowBnd, [], [], [], 'GMMMPEC_c', 'GMMMPEC_dc', [], ConsPattern, c_L, c_U);
    else
	    Prob = conAssign('GMMMPEC_f', [], 'GMMMPEC_hess', HessPattern, x_L, x_U, Name, x00,... 
        	[], fLowBnd, [], [], [], 'GMMMPEC_c', [], [], ConsPattern, c_L, c_U);
    end
    
    %Prob.PriLevOpt = 3; 
    %Prob.KNITRO.PrintFile = ''; 
    Prob.KNITRO.options.ALG = 2;
    Prob.KNITRO.options.MAXIT = 100;    % Setting maximum number of iterations     
    Prob.KNITRO.options.HESSOPT = 4;    % to be used along with first-order derivatives    
    
    Prob.KNITRO.options.OPTTOL = tol_outer ;    % set outer-loop tolerance
    
    MPEC_Result = tomRun('knitro', Prob, 1);
    
    MPECx = MPEC_Result.x_k;
    G = MPEC_Result.f_k;
    INFO = MPEC_Result.Inform;
    CPUt = MPEC_Result.CPUtime;
    FuncEval = MPEC_Result.FuncEv;
    GradEval = MPEC_Result.GradEv; 
    theta1 = MPECx(1:K+1);
    theta2 = MPECx(K+2:2*K+2);
    
    x00 = MPECx;
    reps = reps+1;
end

Dg = zeros(2*K+2);
if deriv==1,
    Dg = mkderivGMMnorm(theta1,theta2,1.e-5);
    cov = inv(Dg*W*Dg');
end

disp(['Monte Carlo Replication ' num2str(rr)])
disp(' ')
