
% June 27, 2008
% Takes many output of JPs script file for many starting values and finds
% one with lowest GMM objective function value

global tol_inner

reps = 100

betatrue = [.1 1.5 1.5 .5 -3]';                     % true mean tastes
K = size(betatrue,1)-1;    

prods = 25;
T = 75;
weak = 4;                                           % weakness of the instruments
sims = 1;                                           % # replications per experiment
BLP=1;
MPEC=1;
elasdo=1;
outputtable = [];
rand('seed',50000);                                 % re-set uniform random number generator
theta0 = rand(K+1,reps)*7;



G_Tight = zeros(reps,1); %GMM objective function values
G_NFPLoose = zeros(reps,1);
G_NFPLoose1 = zeros(reps,1);
G_MPEC = zeros(reps,1);


theta_Tight = zeros(reps,10);  %Structural parameter estimates
theta_NFPLoose = zeros(reps,10);
theta_NFPLoose1 = zeros(reps,10);
theta_MPEC = zeros(reps,10);


elas_Tight = zeros(reps,prods,prods); % Elasticities
elas_NFPLoose = zeros(reps,prods,prods);
elas_NFPLoose1 = zeros(reps,prods,prods);
elas_MPEC = zeros(reps,prods,prods);


info_Tight = zeros(reps,1);
info_Loose = zeros(reps,1);
info_Loose1 = zeros(reps,1);
info_MPEC = zeros(reps,1);


%theta0BLP

for jj = 1:reps,
    load (['MCoutNFPtight_' num2str(T) 'mkts_' num2str(prods) 'products' 'startingvalue' num2str(jj)]);
    load (['MCoutNFPloose_' num2str(T) 'mkts_' num2str(prods) 'products_'  'startingvalue' num2str(jj)]);
    load (['MCoutNFPloose1_' num2str(T) 'mkts_' num2str(prods) 'products_' 'startingvalue' num2str(jj)]);
    load (['MCoutMPEC_' num2str(T) 'mkts_' num2str(prods) 'products_' 'startingvalue' num2str(jj)]);

    G_Tight(jj) = Jstat1tight(1);  %GMM objective function values
    G_NFPLoose(jj) = Jstat1loose(1);
    G_NFPLoose1(jj) = Jstat1loose1(1);
    G_MPEC(jj) = JstatGMMMPEC(1);
    
    theta_Tight(jj,:) = thetaGMM1tight(:,1);  %Structural parameter estimates
    theta_NFPLoose(jj,:) = thetaGMM1loose(:,1);
    theta_NFPLoose1(jj,:) = thetaGMM1loose1(:,1);
    theta_MPEC(jj,:) = thetaGMMMPEC(:,1);
    
    elas_Tight(jj,:,:) = ElasbarBLP;  %Elasticities parameter estimates
    elas_NFPLoose(jj,:,:) = ElasbarBLPloose;
    elas_NFPLoose1(jj,:,:) = ElasbarBLPloose1;
    elas_MPEC(jj,:,:) = ElasbarMPEC;

    info_Tight(jj) = INFOGMM1tight(1);
    info_Loose(jj) = INFOGMM1loose(1);
    info_Loose1(jj) = INFOGMM1loose1(1);
    info_MPEC(jj) = INFOGMMMPEC(1);
    
end


[G_Tight_min G_Tight_min_pos] = min(G_Tight)  %GMM objective function values
[G_NFPLoose_min G_NFPLoose_min_pos] = min(G_NFPLoose)
[G_NFPLoose1_min G_NFPLoose1_min_pos] = min(G_NFPLoose1)
[G_MPEC_min G_MPEC_min_pos] = min(G_MPEC)

theta_Tight_min = theta_Tight(G_Tight_min_pos,:)';  %Structural parameter estimates
theta_NFPLoose_min = theta_NFPLoose(G_NFPLoose_min_pos,:)';
theta_NFPLoose1_min = theta_NFPLoose1(G_NFPLoose1_min_pos,:)';
theta_MPEC_min = theta_MPEC(G_MPEC_min_pos,:)';


elas_Tight_min = reshape(elas_Tight(G_Tight_min_pos,:,:),prods,prods); % Elasticities
elas_NFPLoose_min = reshape(elas_NFPLoose(G_NFPLoose_min_pos,:,:),prods,prods);
elas_NFPLoose1_min = reshape(elas_NFPLoose1(G_NFPLoose1_min_pos,:,:),prods,prods);
elas_MPEC_min = reshape(elas_MPEC(G_MPEC_min_pos,:,:),prods,prods);

elas_Tight_min_ownmean = mean(diag(elas_Tight_min)); % Mean of own price elasticity
elas_NFPLoose_min_ownmean = mean(diag(elas_NFPLoose_min));
elas_NFPLoose1_min_ownmean = mean(diag(elas_NFPLoose1_min));
elas_MPEC_min_ownmean = mean(diag(elas_MPEC_min));


% mean elasticity for all runs

elas_Tight_all_ownmean = zeros(reps,1);
elas_NFPLoose_all_ownmean = zeros(reps,1);
elas_NFPLoose1_all_ownmean = zeros(reps,1);
elas_MPEC_all_ownmean = zeros(reps,1);



for jj=1:reps,
    elas_Tight_all_ownmean(jj) = mean(diag(reshape(elas_Tight(jj,:,:),prods,prods) ) );
    elas_NFPLoose_all_ownmean(jj) = mean(diag(reshape(elas_NFPLoose(jj,:,:),prods,prods) ) );
    elas_NFPLoose1_all_ownmean(jj) = mean(diag(reshape(elas_NFPLoose1(jj,:,:),prods,prods) ) );
    elas_MPEC_all_ownmean(jj) = mean(diag(reshape(elas_MPEC(jj,:,:),prods,prods) ) );  
end

elas_Tight_all_ownmean_mean = mean(elas_Tight_all_ownmean);
elas_NFPLoose_all_ownmean_mean = mean(elas_NFPLoose_all_ownmean);
elas_NFPLoose1_all_ownmean_mean = mean(elas_NFPLoose1_all_ownmean);
elas_MPEC_all_ownmean_mean = mean(elas_MPEC_all_ownmean);

elas_Tight_all_ownmean_sd = std(elas_Tight_all_ownmean);
elas_NFPLoose_all_ownmean_sd = std(elas_NFPLoose_all_ownmean);
elas_NFPLoose1_all_ownmean_sd = std(elas_NFPLoose1_all_ownmean);
elas_MPEC_all_ownmean_sd = std(elas_MPEC_all_ownmean);







% Calculate GMM objective function using true inner loop

tol_inner=1.e-14;
G_Tight_min_full = gmmobjnormsnopt(theta_Tight_min(6:10)) / (prods*T);
G_NFPLoose_min_full = gmmobjnormsnopt(theta_NFPLoose_min(6:10)) / (prods*T);
G_NFPLoose1_min_full = gmmobjnormsnopt(theta_NFPLoose1_min(6:10)) / (prods*T);
G_MPEC_min_full = gmmobjnormsnopt(theta_MPEC_min(6:10)) / (prods*T);
G_Truth_min_full = gmmobjnormsnopt(thetatrue(6:10)) / (prods*T);


% Calculate GMM objective function using loose inner loop

tol_inner=1.e-4;
G_Tight_min_loosecalc = gmmobjnormsnopt(theta_Tight_min(6:10)) / (prods*T);
G_NFPLoose_min_loosecalc = gmmobjnormsnopt(theta_NFPLoose_min(6:10)) / (prods*T);
G_NFPLoose1_min_loosecalc = gmmobjnormsnopt(theta_NFPLoose1_min(6:10)) / (prods*T);
G_MPEC_min_loosecalc = gmmobjnormsnopt(theta_MPEC_min(6:10)) / (prods*T);
G_Truth_min_loosecalc = gmmobjnormsnopt(thetatrue(6:10)) / (prods*T);



jj=1;  % get true elasticity
load(['MCoutBLP_' num2str(T) 'mkts_' num2str(prods) 'products_' 'startingvalue' num2str(jj)]);
Elasbartrue_ownmean = mean(diag(Elasbartrue) );


% fraction of runs within 1% of minimum value from tight NFP


G_Tight_div = (abs(G_Tight / G_Tight_min)<1.01);
G_Tight_div_count = sum(G_Tight_div);

G_NFPLoose_div = (abs(G_NFPLoose / G_Tight_min)<1.01);
G_NFPLoose_div_count = sum(G_NFPLoose_div);

G_NFPLoose1_div = (abs(G_NFPLoose1 / G_Tight_min)<1.01);
G_NFPLoose1_div_count = sum(G_NFPLoose1_div);

G_MPEC_div = (abs(G_MPEC / G_Tight_min)<1.01);
G_MPEC_div_count = sum(G_MPEC_div);


% fracion with info = 0

info_Tight_count0 = sum( (info_Tight==0) )/ reps;
info_Loose_count0 = sum( (info_Loose==0) )/ reps;
info_Loose1_count0 = sum( (info_Loose1==0) )/ reps;
info_MPEC_count0 = sum( (info_MPEC==0) )/ reps;



table_G_allruns = [G_Tight G_NFPLoose G_NFPLoose1 G_MPEC]



tableparams = [theta_Tight_min theta_NFPLoose_min theta_NFPLoose1_min theta_MPEC_min thetatrue]

tableJstat = [G_Tight_min G_NFPLoose_min G_NFPLoose1_min G_MPEC_min]

tableJstat_tightcalc = [G_Tight_min_full G_NFPLoose_min_full G_NFPLoose1_min_full G_MPEC_min_full G_Truth_min_full]

tableJstat_loosecalc = [G_Tight_min_loosecalc G_NFPLoose_min_loosecalc G_NFPLoose1_min_loosecalc G_MPEC_min_loosecalc G_Truth_min_loosecalc]

elas_table = [elas_Tight_min_ownmean elas_NFPLoose_min_ownmean elas_NFPLoose1_min_ownmean elas_MPEC_min_ownmean Elasbartrue_ownmean]

tableFractionWithin1P = [G_Tight_div_count  G_NFPLoose_div_count G_NFPLoose1_div_count G_MPEC_div_count] / reps

tableFractionExit0 = [info_Tight_count0 info_Loose_count0 info_Loose1_count0 info_MPEC_count0]


tableAllRunsMeanElas = [elas_Tight_all_ownmean_mean elas_NFPLoose_all_ownmean_mean elas_NFPLoose1_all_ownmean_mean elas_MPEC_all_ownmean_mean]

tableAllRunsSDElas = [elas_Tight_all_ownmean_sd elas_NFPLoose_all_ownmean_sd elas_NFPLoose1_all_ownmean_sd elas_MPEC_all_ownmean_sd]






% one dimensional plot of GMM objective function


x = 0:0.05:1;
plotpoints = size(x,2);
y_loose = zeros(plotpoints,1);
y_tight = zeros(plotpoints,1);

for k=1:plotpoints
    theta_sd_plot = [thetatrue(6:9)' x(k) ];
    tol_inner=1.e-4;
    y_loose(k) = gmmobjnormsnopt(theta_sd_plot) / (prods*T);
    tol_inner=1.e-14;
    y_tight(k) = gmmobjnormsnopt(theta_sd_plot) / (prods*T);
end









