function [xlow,xupp,Flow,Fupp] = GMMMPECSetup3(x0)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Defines the problem hexagon:
%   maximize the objective row
%   subject to
%            xlow <=   x  <= xupp
%            Flow <= F(x) <= Fupp
%   where
%   
%   x = [theta1; theta2; delta; g];
%
%   F = [G ; 
%        logEstShares ;
%        cong ];
%  
% with 
% 
% cong = g - IV'*(delta - x*theta1);
% G = g'*W*g;
% logEstShares = log(ind_shnorm(expmval,expmu));
% 
% MU = mufuncnormMPEC(theta2);
% expmu = exp(MU); 
% expmval = exp(delta);
% logEstShares = log(ind_shnorm(expmval,expmu));

global share IV K prods T Dthet
global cholesky setuprc

theta1 = x0(1:prods+1, 1);
theta2 = x0(prods+2:prods+K+Dthet+2, 1);
delta = x0(prods+K+Dthet+3:prods+K+Dthet+2+T*prods, 1);
g = x0(prods+K+Dthet+3+T*prods:end, 1);

nIV = size(IV, 2);

neF    = 1+prods*T + nIV;
% neF    = 1+prods*T ;
n      = size(x0, 1);
Obj    = 1; % The default

% Ranges for F.

Flow = zeros(neF,1);
Fupp = zeros(neF,1);

Flow(2:prods*T+1) = log(share);
Fupp(2:prods*T+1) = log(share);

% The Objective row is always free.

Flow(Obj)  = -Inf;
Fupp(Obj)  =  Inf;

xlow = -Inf*ones(n,1);
xupp =  Inf*ones(n,1);

% xlow(prods+2:prods+K+Dthet+2) = 0;

% xlow(prods+2:prods+K+2) = 0;    % constrain r.c.s to be positive in case with independent r.c.s
