function [dc] = GMMMPEC_dc(x0)

% GMMMPEC_dc
% Gradient of the constraints of the random coefficients Logit estimated
% via MPEC.
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global x1 x2 IV K prods v nn T D Dthet keepdem vfull dfull

theta1 = x0(1:prods+1, 1);
theta2 = x0(prods+2:prods+K+Dthet+2, 1);
delta = x0(prods+K+Dthet+3:prods+K+Dthet+2+T*prods, 1);
g = x0(prods+K+Dthet+3+T*prods:end, 1);

theta = theta2(1:K+1);
thetadem = theta2(K+2:end);
thetadem1 = zeros(D);
thetadem1(keepdem) = thetadem;

MU = mufuncnormMPEC(theta2);
expmu = exp(MU); 
expmval = exp(delta);
[EstShare, simShare] = ind_shnormMPEC(expmval,expmu);

nx0 = size(x0,1);
ng = size(g,1);

ooo = ones(1,K+1);
ooo1 = ones(prods,1);
dSdtheta2 = zeros(T*prods,(K+1)*(D+1));
dSddeltaDIAG = zeros(T*prods,prods);
dSddelta = zeros(T*prods, T*prods);

for tt=1:T,
    index = (0:prods-1)'*T+tt;
    for rr = 1:nn,
        dSddeltaDIAG(index,:) = dSddeltaDIAG(index,:) + (diag(simShare(index,rr)) - simShare(index,rr)*simShare(index,rr)')/nn;
       
        v_i = vfull(:,rr:nn:(K+1)*nn);
        d_i = dfull(:,rr:nn:D*nn);
        dSdtheta2(index,1:K+1) = dSdtheta2(index,1:K+1) + (simShare(index,rr)*ooo).*(v_i(index,:)).*( x2(index,:) - (ooo1*(simShare(index,rr)'*x2(index,:))))/nn;
        for d = 1:D
            dSdtheta2(index,1+d*4:(d+1)*4) = dSdtheta2(index,d*4+1:(d+1)*4) + (simShare(index,rr)*ooo).*(d_i(index,d)*ooo).*( x2(index,:) - (ooo1*(simShare(index,rr)'*x2(index,:))))/nn;
        end
    end

    % size(ooo)
    % size(dSdtheta2(index,:))
    % size(EstShare(index)*ooo)
    
    dSdtheta2(index,:) = dSdtheta2(index,:) ./ (EstShare(index)*ones(1,20));
    
    dSddeltaDIAG(index,:) = dSddeltaDIAG(index,:) ./ (EstShare(index)*ooo1');
end

dSddelta = zeros(T*prods, T*prods);

for i = 1:prods
    for j = 1:prods
    dSddelta( (j-1)*T+1:j*T, (i-1)*T+1:i*T ) = diag(dSddeltaDIAG((j-1)*T+1:j*T, i)); 
    end
end

dc1 = [zeros(T*prods,prods+1), dSdtheta2(:,[(1:4)';4+keepdem]'), dSddelta, zeros(T*prods,ng)];

dc2 = [IV'*x1, zeros(ng, (K+1)+Dthet), -IV', eye(ng)];

% dc = dc1;

dc = [dc1; dc2];
