function PROB = ind_shnorm(expmval,expmu)

% IND_SHNORM
% This function computes the "individual" probabilities of choosing each brand.
% The probabilities are those associated with the normally-distributed r.c. logit.
%
% ARGUMENTS:
% expmval = vector of exponentiated mean utilities, sorted by product then market
% expmu = matrix of exponentiated draws of deviations from mean utilities, sorted by product then market
%
% OUTPUT:
% PROB = vector of expected market shares, sorted by product then market
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global oo sharesum denomexpand SS PROB

mval = (expmval*oo ).*expmu;            % this is the numerator (oo speeds-up expanding mean utility by number of draws)
sum1 = sharesum*mval;
sum11 = 1./(1+sum1);                    % this is the denominator of the shares
denom1 = sum11(denomexpand,:);          % this expands the denominator
SS = mval.*denom1;                      % simulated shares for each draw
PROB = mean(SS,2);                      % expected share (i.e. mean across draws)
