function E = mkelas(expmval,expmu,alpha)

% mkelas
%
% L = mkelas(expmval,expmu,alpha) computes the price elasticity matrix
% taking the exponentiated mean utilities, expmval, "nn" draws of exponentiated
% deviations from mean utilities, expmu, and "nn" draws of the price
% coefficient.  The elasticities are computed for all markets in the data.
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008


global T prods nn x1 x2
global oo sharesum denomexpand

mval = (expmval*oo ).*expmu;           % this is the numerator (oo speeds-up expanding mean utility by number of draws)
sum1 = sharesum*mval;
sum11 = 1./(1+sum1);                   % this is the denominator of the shares
denom1 = sum11(denomexpand,:);         % this expands the denominator
S = (expmval*oo).*expmu.*denom1;       % simulated shares for each draw
sh = mean(S,2);                        % expected share (i.e. mean across draws)

E = zeros(T*prods,prods);
for tt = 1:T,
    index = (0:prods-1)*T + tt;
    scale = repmat(x1(index,1),1,prods)./repmat(sh(index)',prods,1)/nn;
    for ii = 1:nn,
        SS = S(index,ii);
        E((tt-1)*prods+1:tt*prods,:) = E((tt-1)*prods+1:tt*prods,:) + alpha(end,ii)*(diag(SS) - SS*SS');
    end
    E((tt-1)*prods+1:tt*prods,:) = E((tt-1)*prods+1:tt*prods,:).*scale;
end
