function [theta1,theta2,G,INFO,CPUt,FuncEval,GradEval,Dg,cov] = runGMMMPECTomlab(x00,deriv,gradient)

% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global x1 iteration W K T prods IV tol_outer rr Dthet

if nargin<3, gradient=0; end;
if nargin<2, deriv=0; end;

iteration = 0;
INFO = -1;
reps =0;
CPUt = 0;
differ = 1;

theta1 = x00(1:prods+1, 1);
theta2 = x00(prods+2:prods+K+Dthet+2, 1);
delta = x00(prods+K+Dthet+3:prods+K+Dthet+2+T*prods, 1);
g = x00(prods+K+Dthet+3+T*prods:end, 1);

nx0 = size(x00,1);
ng = size(g,1);
nIV = size(IV,2);

while INFO == -1 && reps <= 1,
    
    % Setup the lower and upper bounds for varialbes and constraints
    [xlow,xupp,Flow,Fupp] = GMMMPECSetup3(x00);
    
    x_L = xlow ;   %% % Lower bounds for x.
    x_U = xupp ;   %% % Upper bounds for x.
    
    c_L = Flow(2:end);  % Lower bound on nonlinear constraints.
    c_U = Fupp(2:end);  % Upper bound on nonlinear constraints. 
    
    c = GMMMPEC_c(x00);
    f = GMMMPEC_f(x00);
    
    % nx0 = size(x00,1);
    HessPattern = zeros(nx0,nx0);
    HessPattern(prods+K+Dthet+3+T*prods:end,prods+K+Dthet+3+T*prods:end) = 1;
    
    % nIV = size(IV,2);
    
    % c11 = zeros(T*prods, prods+1);
    % c12 = ones(T*prods, K+1+9);
    % c13 = repmat(eye(T), prods, prods);
    % c14 = zeros(T*prods, nIV);
    
    % c21 = ones(nIV, prods+1);    
    % c22 = zeros(nIV, K+1+9);    
    % c23 = ones(nIV, T*prods);
    % c24 = eye(nIV);     
    % ConsPattern = [ c11 c12 c13 c14; c21 c22 c23 c24 ];
    
    % ConsPattern = (dc_x0 ~= 0);
    dc_x0 = GMMMPEC_dc(x00);    
    ConsPattern = (dc_x0 ~= 0);
    % LinCon = [IV'*x1, zeros(ng, (K+1)+Dthet), -IV', eye(ng)];
    % b_L = zeros(ng,1);
    % b_U = b_L;
    
    Name = 'BLP MPEC Problem';
    fLowBnd = 0;         % Lower bound on function.
    
    if gradient==1,
        Prob = conAssign('GMMMPEC_f', 'GMMMPEC_grad', 'GMMMPEC_hess', HessPattern, x_L, x_U, Name, x00,... 
        	[], fLowBnd, [], [], [], 'GMMMPEC_c', 'GMMMPEC_dc', [], ConsPattern, c_L, c_U);
    else
	    Prob = conAssign('GMMMPEC_f', [], 'GMMMPEC_hess', HessPattern, x_L, x_U, Name, x00,... 
        	[], fLowBnd, LinCon, b_L, b_U, 'GMMMPEC_c', [], [], ConsPattern, c_L, c_U);
    end
    
    % checkDerivs(Prob, x00, 1, 1, 1)
    
    Prob.PriLevOpt = 3; 
    Prob.KNITRO.PrintFile = ''; 
    Prob.KNITRO.options.ALG = 2;
    Prob.KNITRO.options.MAXIT = 100;            % Setting maximum number of iterations     
    Prob.KNITRO.options.HESSOPT = 4;            % to be used along with first-order derivatives    
    
    Prob.KNITRO.options.OPTTOL = tol_outer ;    % set outer-loop tolerance
    
    MPEC_Result = tomRun('knitro', Prob, 1);
    
    MPECx = MPEC_Result.x_k;
    G = MPEC_Result.f_k;
    INFO = MPEC_Result.Inform;
    CPUt = MPEC_Result.CPUtime;
    FuncEval = MPEC_Result.FuncEv;
    GradEval = MPEC_Result.GradEv; 
    theta1 = MPECx(1:prods+1);
    theta2 = MPECx(prods+2:(prods+1)+(K+1+9));
    
    x00 = MPECx;
    reps = reps+1;
end

if nargin>=2,
    if deriv==1,
        Dg = mkderivGMMnorm(theta1,theta2,1.e-5);
        cov = inv(Dg*W*Dg');
    else
        Dg = [];
        cov = [];
    end
end

disp(['Monte Carlo Replication ' num2str(rr)])
disp(' ')
