function [thetaout,Lipout] = MC(prodsset,Tset,sims,weak,sigmaxi,BLP,MPEC,NFPlooseinner,NFPlooseboth,elasdo,startingNum,usegradients)

% MC
%
% Executes 'sims' replications of a Monte Carlo experiment for the 
% Normal R.C. Logit model with 'prods' products and 'T' markets 
% BLP = 1, 0 indicates whether to run NFP
% MPEC =1,0 indicates whether to run MPEC
% NFPlooseinner runs a loose inner loop NFP if in addition BLP=1
% NFPlooseboth runs a loose inner loop and optimization routine NFP if in addition BLP=1
% prodsset is number of products per market
% Tset is number of markets
% sims is number of simulation draws
% weak enters the formula for generating the instruments
% sigmaxi is the scale of the additive error
% elasdo should usually be 1
% startingNum is the number of starting values
% usegradients is a flag for using gradient information


%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008, December 2010

global nn K v covX costparams betatrue rc rctrue
global y price share W datasort denomexpand sharesum PX x IV setuprc cholesky
global T prods theta1 oo1 rr tol_inner tol_outer nopurchglobal multistarts
global scale_global nIV nopurch

%%%% INITIALIZE VARIABLES FOR OUTPUT %%%%
prods = prodsset;
T = Tset;

if nargin<9, startingNum=1; end;        
if nargin<10, usegradients=0; end;     


thetaGMM1tight = [];
Jstat1tight    = [];
timeGMM1tight = [];
innerlooptimetight = [];
FuncEvGMM1tight = [];
GradEvGMM1tight = [];
HessEvalGMM1tight = [];
IterGMM1tight = [];
ContractEvalGMM1tight = [];
covGMM1tight = [];
repsGMM1tight = [];
INFOGMM1tight = [];

thetaGMM1loose = [];
Jstat1loose    = [];
timeGMM1loose = [];
innerlooptimeloose = [];
FuncEvGMM1loose = [];
GradEvGMM1loose = [];
covGMM1loose = [];
repsGMM1loose = [];
INFOGMM1loose = [];

thetaGMM1loose1 = [];
Jstat1loose1    = [];
timeGMM1loose1 = [];
innerlooptimeloose1 = [];
FuncEvGMM1loose1 = [];
GradEvGMM1loose1 = [];
covGMM1loose1 = [];
repsGMM1loose1 = [];
INFOGMM1loose1 = [];


thetaGMM2 = [];
Jstat2    = [];
covGMM2 = [];

thetaGMMMPEC = [];
JstatGMMMPEC = [];
timeGMMMPEC = [];
FuncEvGMMMPEC = [];
GradEvGMMMPEC = [];
covGMMMPEC = [];
INFOGMMMPEC = [];
IterGMMMPEC = [];
HessEvalGMMMPEC = [];
           
Elasbartrue = zeros(prods, prods, sims);
ElasbarBLP = zeros(prods, prods, sims);
ElasbarBLPloose = zeros(prods, prods, sims);
ElasbarBLPloose1 = zeros(prods, prods, sims);
ElasbarMPEC = zeros(prods, prods, sims);

Lipschtrue = zeros(sims,1);        
LipschBLP = zeros(sims,1);
LipschBLPloose = zeros(sims,1);
LipschBLPloose1 = zeros(sims,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Create indices to speed share calculations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
oo = ones(1,nn);                                    % index for use in simulation of shares
oo1 = (0:T-1)*prods+prods;                          % indicates last observation for each market
sharesum = kron(speye(T), ones(1,prods));           % used to create denominators in predicted shares (i.e. sums numerators)
datasort = [(1:T*prods)' repmat((1:T)',prods,1) kron( (1:prods)',ones(T,1))];
datasort = datasort(:,1);
denomexpand = kron((1:T)',ones(prods,1));

%%% initialize matrices for output of replications of the experiment%%%
output = [];
betaols = [];
beta2sls = [];
seols = [];
se2sls = [];
drop = [];
rand('seed',500000)                           % reset uniform random number generator
randn('seed',500000)

for rr = 1:sims,

    % CREATE SIMULATED DATA %
    xi = randn( T*prods,1)*sigmaxi;                      % draw demand shocks
    A = [kron(ones(T,1), randn( prods,K-1)*chol(covX))];  % product attributes
    prand = rand( T*prods,1)*5;
    price = 3 +   xi*1.5 +  prand + sum(A,2);
   
    z = rand( T*prods,length(costparams)) + 1/weak*repmat( abs( prand +  sum(A,2)*1.1 ) ,1,length(costparams));
    x = [ones(T*prods,1) A price];
    [share,nopurch] = mksharesim(betatrue,x,xi,rc);
    % y = log(share) - log(repmat(nopurch,prods,1));      % log-odds ratio
    y = log(share) - log(kron(nopurch,ones(prods,1)));      % log-odds ratio
    % y = (share)./(kron(nopurch,ones(prods,1)));      % log-odds ratio
    iv = [ones(prods*T,1) A z];                         % matrix of instruments

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 2SLS ESTIMATION OF HOMOGENEOUS LOGIT MODEL %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    xhat = iv*inv(iv'*iv)*iv'*x;
    PX2sls = inv(xhat'*xhat)*xhat';                     % project. matrix on weighted x-space for 2SLS
    beta2sls = PX2sls*y;
    se2sls = sqrt(diag( mean((y-x*beta2sls).^2)*inv(xhat'*xhat) ));

    expmeanval0 = exp(y);
    IV = [ones(T*prods,1) A z A.^2 A.^3 z.^2 z.^3 prod(A,2) prod(z,2) kron(A(:,1),ones(1,size(z,2))).*z  kron(A(:,2),ones(1,size(z,2))).*z];
    W = inv(IV'*IV);
    PX = inv(x'*IV*W*IV'*x)*x'*IV*W*IV';
    nIV = size(IV,2);  

    
    save 2sls betaols beta2sls seols se2sls
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% STARTING VALUES FOR VARIANCES OF RANDOM COEFFICIENTS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
    %% Create multistarts starting values
    
    theta20_list  = rand(startingNum,length(betatrue));
  
    
    
    
    %%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % GMM ESTIMATION OF R.C. LOGIT (1st step only) %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    cholesky = K;                           % set to K*(K-1) for correlated r.c. case
    setuprc = find(tril(ones(K)));          % to assemble cholesky matrix for case of correlated r.c.s
    mvalold0 = exp(y);
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % (1) run NFP approach%
    if BLP==1,
        
        % (i) RUN TIGHT INNER LOOP AND TIGHT OUTER LOOP
        tol_inner=1.e-14;
        tol_outer=1.e-6;
        
        G = 1.0e20;
        CPUt = 0;
        FuncEval = 0;
        GradEval = 0;
        HessEval = 0;
        Iter = 0;
        ContractEval = 0;
        
        theta = zeros(size(rctrue));  % just in case no convergences
        y_keep = y; % just in case no convergences
        reps_report_keep = 0; % just in case no convergences
        cov = 0; % just in case no convergences 
        theta1_keep = zeros(size(betatrue)); % just in case no convergences
        INFO=-1;  % just in case no convergences
        
        for reps=1:multistarts,
            theta20rep = theta20_list(reps,:)';
            [theta_rep,G_rep,INFO_rep,CPUt_rep,FuncEval_rep,GradEval_rep,reps_report,Dg,cov_rep,Iter_rep,HessEval_rep,ContractEval_rep] = runGMMBLPTomlab(theta20rep,W,mvalold0,1,usegradients);
            
            CPUt = CPUt + CPUt_rep;
            FuncEval = FuncEval + FuncEval_rep;
            GradEval = GradEval + GradEval_rep;
            Iter = Iter + Iter_rep;
            HessEval = HessEval + HessEval_rep;
            ContractEval = ContractEval + ContractEval_rep;
            
            if (G_rep < G && INFO_rep==0),
                theta = theta_rep;
                G = G_rep;
                INFO = INFO_rep;
                gmmblpobj(theta); % to create a theta1 in global memory
                theta1_keep = theta1;
                y_keep = y;
                reps_report_keep = reps_report;
                cov = cov_rep;
            end
            
            disp(['NFP'])
            disp(['Scale ' num2str(scale_global)])
            disp(['Markets ' num2str(Tset)])
            disp(['Monte Carlo Replication ' num2str(rr)])
            disp(['# runs of optimizer     ' num2str(reps)])
            disp(' ')
            
        end
            
        thetaGMM1tight = [ thetaGMM1tight, [theta1_keep;theta]];
        Jstat1tight = [Jstat1tight, G/(T*prods)];
        timeGMM1tight = [timeGMM1tight, CPUt];
        FuncEvGMM1tight = [FuncEvGMM1tight, FuncEval];
        GradEvGMM1tight = [GradEvGMM1tight, GradEval]; 
        HessEvalGMM1tight = [HessEvalGMM1tight, HessEval];
        IterGMM1tight = [IterGMM1tight, Iter];
        ContractEvalGMM1tight = [ContractEvalGMM1tight, ContractEval];
        covGMM1tight = [covGMM1tight,cov];
        repsGMM1tight = [repsGMM1tight, reps_report_keep];
        INFOGMM1tight = [INFOGMM1tight, INFO];
        
        mvalold0 = exp(y_keep);
        tic; meanvalnorm(rctrue); tim=toc;
        innerlooptimetight = [innerlooptimetight , tim];
        save GMMstep1tight thetaGMM1tight W Jstat1tight timeGMM1tight FuncEvGMM1tight GradEvGMM1tight covGMM1tight repsGMM1tight INFOGMM1tight innerlooptimetight ContractEvalGMM1tight HessEvalGMM1tight IterGMM1tight;

        
        % (ii) RUN LOOSE INNER LOOP AND TIGHT OUTER LOOP
        tol_inner=1.e-4;
        tol_outer=1.e-6;
        mvalold0 = exp(y);
        theta00 = theta20_list(1,:)';
        
        
        if NFPlooseinner==1,
       		[theta,G,INFO,CPUt,FuncEval,GradEval,reps,Dg,cov] = runGMMBLPTomlab(theta00,W,mvalold0,1,usegradients);
       	end
       	
        thetaGMM1loose = [ thetaGMM1loose, [theta1;theta]];
        Jstat1loose = [Jstat1loose, G/(T*prods)];
        timeGMM1loose = [timeGMM1loose, CPUt];
        FuncEvGMM1loose = [FuncEvGMM1loose, FuncEval];
        GradEvGMM1loose = [GradEvGMM1loose, GradEval]; 
        covGMM1loose = [covGMM1loose,cov];
        repsGMM1loose = [repsGMM1loose, reps];
        INFOGMM1loose = [INFOGMM1loose, INFO]; 
        mvalold0 = exp(y);
        tic; meanvalnorm(rctrue); tim=toc;
        innerlooptimeloose = [innerlooptimeloose , tim];
        save GMMstep1loose thetaGMM1loose W Jstat1loose timeGMM1loose FuncEvGMM1loose GradEvGMM1loose covGMM1loose repsGMM1loose INFOGMM1loose innerlooptimeloose;


        % (iii) RUN LOOSE INNER LOOP AND LOOSE OUTER LOOP
        tol_inner=1.e-4;
        tol_outer=1.e-2;
        mvalold0 = exp(y);
        theta00 = theta20_list(1,:)';
        
        if NFPlooseboth==1,
        	[theta,G,INFO,CPUt,FuncEval,GradEval,reps,Dg,cov] = runGMMBLPTomlab(theta00,W,mvalold0,1,usegradients);
       	end
        thetaGMM1loose1 = [ thetaGMM1loose1, [theta1;theta]];
        Jstat1loose1 = [Jstat1loose1, G/(T*prods)];
        timeGMM1loose1 = [timeGMM1loose1, CPUt];
        FuncEvGMM1loose1 = [FuncEvGMM1loose1, FuncEval];
        GradEvGMM1loose1 = [GradEvGMM1loose1, GradEval]; 
        covGMM1loose1 = [covGMM1loose1,cov];
        repsGMM1loos1e = [repsGMM1loose1, reps];
        INFOGMM1loose1 = [INFOGMM1loose1, INFO]; 
        mvalold0 = exp(y);
        tic; meanvalnorm(rctrue); tim=toc;
        innerlooptimeloose1 = [innerlooptimeloose1 , tim];
        save GMMstep1loose1 thetaGMM1loose1 W Jstat1loose1 timeGMM1loose1 FuncEvGMM1loose1 GradEvGMM1loose1 covGMM1loose1 repsGMM1loose1 INFOGMM1loose1 innerlooptimeloose1;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%
    % (2) run MPEC approach%
    if MPEC==1,
        tol_outer=1.e-6;
        
        GMPEC = 1.0e20;  % high baseline value as a dummy argument
        CPUtMPEC = 0;
        FuncEvalMPEC = 0;
        GradEvalMPEC = 0;
        HessEvalMPEC = 0;
        IterMPEC = 0;
        
        theta1MPEC = zeros(size(betatrue));  % just in case no convergences
        theta2MPEC = zeros(size(rctrue));; % just in case no convergences
        covMPEC = 0; % just in case no convergences
        INFOMPEC = -2; % just in case no convergences
        
        for reps=1:multistarts,
            
            theta20rep = theta20_list(reps,:)';
            delta0 = meanvalnorm(theta20rep);
            theta10 = PX*delta0;
            resid0 = delta0 - x*theta10;
            g0 = IV'*resid0;
            x0 = [theta10; theta20rep; delta0; g0];
            
          
            
            [theta1MPEC_rep,theta2MPEC_rep,GMPEC_rep,INFOMPEC_rep,CPUtMPEC_rep,FuncEvalMPEC_rep,GradEvalMPEC_rep,DgMPEC_rep,covMPEC_rep,delta_rep,Iter_rep,HessEval_rep] = runGMMMPECTomlab(x0,usegradients);
            
            CPUtMPEC = CPUtMPEC + CPUtMPEC_rep;
            FuncEvalMPEC = FuncEvalMPEC + FuncEvalMPEC_rep;
            GradEvalMPEC = GradEvalMPEC + GradEvalMPEC_rep;
            IterMPEC = IterMPEC + Iter_rep;
            HessEvalMPEC = HessEvalMPEC + HessEval_rep;
            
            if (GMPEC_rep < GMPEC && INFOMPEC_rep==0),
                theta1MPEC = theta1MPEC_rep;
                theta2MPEC = theta2MPEC_rep;
                GMPEC = GMPEC_rep;
                INFOMPEC = INFOMPEC_rep;
                covMPEC = covMPEC_rep;
            end
            
            disp(['MPEC'])
            disp(['Scale ' num2str(scale_global)])
            disp(['Markets ' num2str(Tset)])
            disp(['Monte Carlo Replication ' num2str(rr)])
            disp(['# runs of optimizer     ' num2str(reps)])
            disp(' ')
            
        end
        
        
        
        thetaGMMMPEC = [ thetaGMMMPEC, [theta1MPEC;theta2MPEC]];
        JstatGMMMPEC = [JstatGMMMPEC, GMPEC/(T*prods)];
        timeGMMMPEC = [timeGMMMPEC, CPUtMPEC];
        FuncEvGMMMPEC = [FuncEvGMMMPEC, FuncEvalMPEC]; 
        GradEvGMMMPEC = [GradEvGMMMPEC, GradEvalMPEC]; 
        covGMMMPEC = [covGMMMPEC,covMPEC];
        INFOGMMMPEC = [INFOGMMMPEC, INFOMPEC];
        HessEvalGMMMPEC = [HessEvalGMMMPEC, HessEvalMPEC]; 
        IterGMMMPEC = [IterGMMMPEC, IterMPEC];
        
        save GMMMPEC thetaGMMMPEC JstatGMMMPEC timeGMMMPEC FuncEvGMMMPEC GradEvGMMMPEC covGMMMPEC INFOGMMMPEC IterGMMMPEC HessEvalGMMMPEC;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% COMPARE ECONOMIC METRICS %%%%
    deltatrue = x*betatrue+xi;
    Lipschitztrue = mklipschitz(exp(deltatrue),exp(x*rc));
    Lipschtrue(rr,1) = Lipschitztrue;
    if elasdo==1,
        alphatrue = betatrue(end) + rc(end,:);
        Elastrue = mkelas(exp(deltatrue),exp(x*rc),alphatrue);
    end
    
    if BLP==1,
        thetaBLP1 = thetaGMM1tight(1:K+1, rr);
        thetaBLP2 = thetaGMM1tight(K+2:end, rr);
        deltaBLP = meanvalnorm(thetaBLP2);        % invert out the mean utilities using BLP tastes
        rcBLP = diag(thetaBLP2)*v;
        LipschitzBLP = mklipschitz(exp(deltaBLP),exp(x*rcBLP));
        LipschBLP(rr,1) = LipschitzBLP;
        if elasdo==1,
            alphaBLP = thetaBLP1(end) + rcBLP(end,:);
            ElasBLP = mkelas(exp(deltaBLP),exp(x*rcBLP),alphaBLP);
        end
        
        
        thetaBLP1loose = thetaGMM1loose(1:K+1, rr);
        thetaBLP2loose = thetaGMM1loose(K+2:end, rr);
        deltaBLPloose = meanvalnorm(thetaBLP2loose);        % invert out the mean utilities using BLP tastes
        rcBLPloose = diag(thetaBLP2loose)*v;
        LipschitzBLPloose = mklipschitz(exp(deltaBLPloose),exp(x*rcBLPloose));
        LipschBLPloose(rr,1) = LipschitzBLPloose;
        if elasdo==1,
            alphaBLPloose = thetaBLP1loose(end) + rcBLPloose(end,:);
            ElasBLPloose = mkelas(exp(deltaBLPloose),exp(x*rcBLPloose),alphaBLPloose);
        end

        thetaBLP1loose1 = thetaGMM1loose1(1:K+1, rr);
        thetaBLP2loose1 = thetaGMM1loose1(K+2:end, rr);
        deltaBLPloose1 = meanvalnorm(thetaBLP2loose1);        % invert out the mean utilities using BLP tastes
        rcBLPloose1 = diag(thetaBLP2loose1)*v;
        LipschitzBLPloose1 = mklipschitz(exp(deltaBLPloose1),exp(x*rcBLPloose1));
        LipschBLPloose1(rr,1) = LipschitzBLPloose1;
        if elasdo==1,
            alphaBLPloose1 = thetaBLP1loose1(end) + rcBLPloose1(end,:);
            ElasBLPloose1 = mkelas(exp(deltaBLPloose1),exp(x*rcBLPloose1),alphaBLPloose1);
        end
    end
    
    if MPEC==1,
        thetaMPEC1 = thetaGMMMPEC(1:K+1, rr);
        thetaMPEC2 = thetaGMMMPEC(K+2:end, rr);
        deltaMPEC = meanvalnorm(thetaMPEC2);        % invert out the mean utilities using MPEC tastes
        rcMPEC = diag(thetaMPEC2)*v;
        if elasdo==1,
            alphaMPEC = thetaMPEC1(end) + rcMPEC(end,:);
            ElasMPEC = mkelas(exp(deltaMPEC),exp(x*rcMPEC),alphaMPEC);
        end
    end
    
    if elasdo==1,
        for aa = 1:prods,
            index = (0:T-1)*prods + aa;
            Elasbartrue(aa,:, rr) = mean(Elastrue(index,:) );
            save Elasbartrue Elasbartrue
            if BLP==1,  
                ElasbarBLP(aa,:, rr) = mean(ElasBLP(index,:) );
                save ElasbarBLP ElasbarBLP
                
                ElasbarBLPloose(aa,:, rr) = mean(ElasBLPloose(index,:) );
                save ElasbarBLPloose ElasbarBLPloose

                ElasbarBLPloose1(aa,:, rr) = mean(ElasBLPloose1(index,:) );
                save ElasbarBLPloose1 ElasbarBLPloose1

            end;
            if MPEC==1, 
                ElasbarMPEC(aa,:, rr) = mean(ElasMPEC(index,:) );
                save ElasbarMPEC ElasbarMPEC
            end;
        end;
    end

end

thetaout = [];
if BLP==1, thetaout = [thetaout,thetaGMM1tight]; end;
if MPEC==1,thetaout = [thetaout,thetaGMMMPEC]; end;

Lipout = Lipschtrue;
if BLP==1, Lipout = [Lipout,LipschBLP]; end;
