function [Lipschitz LipschitzL2] = mklipschitz(expmval,expmu)

% mklipschitz
%
% L = mklipschitz(expmval,expmu) computes the Lipschitz contant for the BLP
% contraction-mapping taking the exponentiated mean utilities, expmval, and "nn" draws of exponentiated
% deviations from mean utilities, expmu.  This can be used to measure the
% exact rate of convergence for the inner loop in the NFP estimator proposed in BLP.
% see Dube, Fox and Su (2008) for the derivation.
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global T prods nn K
global oo sharesum denomexpand

mval = (expmval*oo ).*expmu;           % this is the numerator (oo speeds-up expanding mean utility by number of draws)
sum1 = sharesum*mval;
sum11 = 1./(1+sum1);                   % this is the denominator of the shares
denom1 = sum11(denomexpand,:);         % this expands the denominator
S = mval.*denom1;       % simulated shares for each draw
sh = mean(S,2);                        % expected share (i.e. mean across draws)

ooo = ones(1,K+1);
ooo1 = ones(prods,1);

dSddeltaDIAG = zeros(T*prods,prods);
dSddelta = zeros(T*prods, T*prods);


for tt=1:T,
    index = (1:prods)'+(tt-1)*prods;  
    for ii = 1:nn,
        SS = S(index,ii);
        dSddeltaDIAG(index,:) = dSddeltaDIAG(index,:) + (diag(SS) - SS*SS')/nn;
    end
    
    dSddeltaDIAG(index,:) = dSddeltaDIAG(index,:) ./ (sh(index)*ooo1');
end

dSddelta = zeros(T*prods, T*prods);


for tt=1:T,
    index = (1:prods)'+(tt-1)*prods; 
    for i=1:prods
        for j=1:prods
            dSddelta( index(j), index(i) ) = dSddeltaDIAG(index(j),i);
        end
    end
end

L = eye(T*prods) - dSddelta;
Lipschitz = norm(L,inf);
LipschitzL2 = mean(mean(L));


