function [theta1,theta2,G,INFO,CPUt,FuncEval,GradEval, d2L, lambda, MPECx] = runGMMMPECTomlab(x00,gradient)

% source: Dube, Fox and Su (2009)
% Code Revised: March 2009

global iteration theta1 K T prods IV tol_outer share nIV x ConsPattern d2LPatt

if nargin<2,
    gradient=0;
end

iteration = 0;
INFO = -1;
reps =0;
CPUt = 0;
differ = 1;
while INFO == -1 && reps <= 1,
    
    nx0 = size(x00,1);
    x_L = -Inf*ones(nx0,1);   % Lower bounds for x.
    x_L(6:10) = 0;
    x_U =  Inf*ones(nx0,1);   % Upper bounds for x.
        
    neF    = prods*T + nIV;
    % neF    = prods*T;
    c_L = zeros(neF,1);
    c_U = zeros(neF,1);
    
    c_L(1:prods*T) = share;
    c_U(1:prods*T) = share;
    
    disp('Constructing sparsity pattern of Hessian of objective')
    
    a = repmat(2*K+2+T*prods+(1:nIV)',nIV,1);
    b = kron(2*K+2+T*prods+(1:nIV)', ones(nIV,1));
    s = ones(nIV*nIV,1);
    HessPattern = sparse(a,b,s,nx0,nx0);
        
    disp('Constructing sparsity pattern of constraint Jacobian')
    
    a1 = repmat((1:T*prods)', K+1,1);
    b1 = kron((K+2:2*K+2)',ones(T*prods,1));
    s1 = ones(T*prods*(K+1),1);
        
    a2 = zeros(T*prods^2,1);
    b2 = zeros(T*prods^2,1);
    s2 = ones(T*prods^2,1);
    for tt=1:T,
        index = (1:prods)'+(tt-1)*prods;
        a2((1:prods^2)'+(tt-1)*prods^2) = repmat(index,prods,1);
        b2((1:prods^2)'+(tt-1)*prods^2) = kron(2*K+2+index,ones(prods,1));
    end        % specify sparsity pattern for constraints
    
    a3 = repmat((T*prods+1:T*prods+nIV)',K+1+T*prods,1);
    b3 = kron([(1:K+1)'; (2*K+3:2*K+2+T*prods)'], ones(nIV,1));   
    s3 = ones(length(a3),1);
    
    a4 = (T*prods+1:T*prods+nIV)';
    b4 = (2*K+2+T*prods+1:2*K+2+T*prods+nIV)';
    s4 = ones(nIV,1);
    aCons = [a1 ; a2; a3; a4];
    bCons = [b1 ; b2; b3; b4];
    sCons = [s1 ; s2; s3; s4];
    ConsPattern = sparse(aCons, bCons, sCons, T*prods+nIV, nx0);
    
    disp('Constructing sparsity pattern of Hessian of the Lagragian')
    
    a5 = repmat((K+2:2*K+2)',K+1,1);
    b5 = kron((K+2:2*K+2)', ones(K+1,1));
    
    a6 = a1+2*K+2;
    b6 = b1;
    a7 = a2+2*K+2;
    b7 = b2;
    
    ad2L = [a5;a6;b6;a7;a];
    bd2L = [b5;b6;a6;b7;b];
    sd2L = ones(T*prods^2+nIV^2+(K+1)^2+2*(K+1)*T*prods,1);
    
    d2LPatt = sparse(ad2L, bd2L, sd2L, nx0, nx0);
    disp('END of Constructing sparsity pattern of Hessian of the Lagragian')
    
    Name = 'BLP MPEC Problem';
    fLowBnd = 0;                                                % Lower bound on function.

    if gradient==1,
        Prob = conAssign('GMMMPEC_f', 'GMMMPEC_grad', 'GMMMPEC_hess', HessPattern, x_L, x_U, Name, x00,... 
        	[], fLowBnd, [], [], [], 'GMMMPEC_c', 'GMMMPEC_dc', 'GMMMPEC_d2c', ConsPattern, c_L, c_U);
    else
	    Prob = conAssign('GMMMPEC_f', [], 'GMMMPEC_hess', HessPattern, x_L, x_U, Name, x00,... 
        	[], fLowBnd, [], [], [], 'GMMMPEC_c', [], [], ConsPattern, c_L, c_U);
    end
    Prob.LargeScale = 1;
    Prob.d2LPattern = d2LPatt;
    Prob.PriLevOpt = 3; 
    Prob.KNITRO.PrintFile = ''; 
    Prob.KNITRO.options.ALG = 1;
    % Prob.KNITRO.options.MAXIT = 200;    % Setting maximum number of iterations 
    Prob.KNITRO.options.MAXIT = 100;    % Setting maximum number of iterations      
   
    Prob.KNITRO.options.OPTTOL = tol_outer ;    % set outer-loop tolerance
    
    MPEC_Result = tomRun('knitro', Prob, 1);
    
    MPECx = MPEC_Result.x_k;
    G = MPEC_Result.f_k;
    d2L = MPEC_Result.H_k;
    lambda = MPEC_Result.v_k;
    INFO = MPEC_Result.Inform;
    CPUt = MPEC_Result.CPUtime;
    FuncEval = MPEC_Result.FuncEv;
    GradEval = MPEC_Result.GradEv; 
    theta1 = MPECx(1:K+1);
    theta2 = MPECx(K+2:2*K+2);

    reps = reps+1;
end
