function [ProjChebpoly,Chebpoly,oo1,oo2,Chebygrid,Chebypolyobs] = Chebyshevsetup2dim(G,K,a1,b1,a2,b2,stateobs)

% CHEBYSHEVSETUP2DIM
%
% Set up the matrices for Chebyshev approximation of a 2-dimensional
% function.
% G = # Chebyshev interpolation nodes (i.e. grid will have G^2 points)
% K = order of the Chebyshev polynomial approximation
% Approximation is defined on the compact set [a1,b1] x [a2,b2]

if G<K, error('Fewer nodes than polynomials'); end

% Create indices to speed up sorting and other calculations
ooK = ones(1,K);
oo1 = kron((1:K),ooK);                              % to expand state variable 1 for tensor product (for Chebyshev regression)
oo2 = repmat((1:K),1,K);                            % to expand state variable 2 for tensor product (for Chebyshev regression)
oo1grid = kron((1:G)',ones(G,1));
oo2grid = repmat((1:G)',G,1);

z = -cos( (2*(1:G)'-1)/(2*G)*pi );                  % create the nodes
x1k = (z+1)*( (b1-a1)/2) + a1;                      % adjust nodes to [a1,b1]
x2k = (z+1)*( (b2-a2)/2) + a2;                      % adjust nodes to [a2.b2]
Chebygrid = [x1k(oo1grid) x2k(oo2grid)];

rhoz = ones(G,K);
rhoz(:,2) = z;
for ii=3:K,
    rhoz(:,ii)  = 2*z.*rhoz(:,ii-1)   - rhoz(:,ii-2);
end
Chebpoly = kron(kron(rhoz,ones(G,1)),ones(1,K)).*repmat(rhoz,G,K);

ProjChebpoly = inv(Chebpoly'*Chebpoly)*Chebpoly';

if nargin==7,
    rhoobs1 = ones(length(stateobs),K);
    rhoobs2 = ones(length(stateobs),K);
    rhoobs1(:,2) = 2*(stateobs(:,1)-a1)/(b1-a1)-1;
    rhoobs2(:,2) = 2*(stateobs(:,2)-a2)/(b2-a2)-1;
    for ii=3:K,
        rhoobs1(:,ii)  = 2*rhoobs1(:,2).*rhoobs1(:,ii-1)   - rhoobs1(:,ii-2);
        rhoobs2(:,ii)  = 2*rhoobs2(:,2).*rhoobs2(:,ii-1)   - rhoobs2(:,ii-2);
    end
    Chebypolyobs = rhoobs1(:,oo1).*rhoobs2(:,oo2);       % tensor product of Chebyshev polynomials at observed states
end