function c = loglik_finmixMPEC_price_MAD_c_2R(theta)

% LOGLIK_FINMIXMPEC_PRICE_MAD_C_2R
%MPEC Constraints for Log-Likelihood function for the Dynamic finite
%mixture random coefficients Logit (only on price coefficient).
% MPEC algorithm used with constraints:
%   1) Bellman's equation
%   2) Shares
%
% source: Dube, Fox and Su (2008)
% Code Revised: May 2008

global prods K Kp Kcov X Mkts T M R ooR sharesum denomexpand run iteration quadpts_estim qwts delta
global ProjChebpoly Chebyorder Chebypolyobs Xstate1 Xstate2 Chebygridpts sims a1 a2 b1 b2

% SET UP PARAMETERS
Theta = reshape(theta([1:K 1:2 K+1]),K,R);              % preferences
muparam = [theta(K+1+1:K+1+(R-1))];                     % mixing parameters
mu = [muparam;1-sum(muparam)];
thetap = theta(K+1+(R-1)+1:K+1+(R-1)+Kp);               % pricing function parameters
thetacov = theta(K+1+(R-1)+Kp+1:K+1+(R-1)+Kp+Kcov);

cholCOVu = zeros(2*prods,2*prods)*mu(1);
cholCOVu(1,1) = thetacov(1);
cholCOVu(2,2) = thetacov(1);
cholCOVu(3,3) = thetacov(3);
cholCOVu(4,4) = thetacov(3);
cholCOVu(1,3) = thetacov(2);
cholCOVu(2,4) = thetacov(2);
COVu = cholCOVu'*cholCOVu;
a = COVu(1,1); b = COVu(1,2); c = COVu(1,3); d = COVu(1,4); 
e = COVu(2,1); f = COVu(2,2); g = COVu(2,3); h = COVu(2,4); 
i = COVu(3,1); j = COVu(3,2); k = COVu(3,3); l = COVu(3,4); 
m = COVu(4,1); n = COVu(4,2); o = COVu(4,3); pp = COVu(4,4);
detCOVu = i*c*h*n-i*c*f*pp+a*f*k*pp-a*h*k*n;
invCOVu = [ [-(k/(-a*k+c^2))	, 0 , (c/(-a*k+c^2))	, 0]
            [0	, -(k/(-a*k+c^2))	, 0 , (c/(-a*k+c^2))]
            [(c/(-a*k+c^2))	, 0 , -(a/(-a*k+c^2))	, 0]
            [0	, (c/(-a*k+c^2))	, 0 , -(a/(-a*k+c^2))]];

qu = sqrt(2)*quadpts_estim*cholCOVu;                                                   % quadrature points to integrate over the distribution of shocks

% EXPECTED VALUE OF WAITING AT OBSERVED STATES AND CHOICE-SPECIFIC VALUE FUNCTIONS
thetaChebyshev01 = theta(K+1+(R-1)+Kp+Kcov+1:K+1+(R-1)+Kp+Kcov+Chebyorder^2);
thetaChebyshev02 = theta(K+1+(R-1)+Kp+Kcov+1+Chebyorder^2:K+1+(R-1)+Kp+Kcov+2*Chebyorder^2);
V0obs1 = Chebypolyobs*thetaChebyshev01;
V0obs2 = Chebypolyobs*thetaChebyshev02;
xi = theta(K+1+(R-1)+Kp+Kcov+R*Chebyorder^2+1:K+1+(R-1)+Kp+Kcov+R*Chebyorder^2+prods*T*Mkts);	% demand shocks
Vjnetxi = X*Theta-repmat([V0obs1 V0obs2],prods,1);                                              % choice-specific value functions


%%%%%%%%%%%%%%%%%%%%%%%
% CONSTRAINTS
%%%%%%%%%%%%%%%%%%%%%%%

% (i) Pre-Compute the distribution of next-period prices and utilities at each state on the grid
state1 = [Xstate1*thetap , Xstate2*thetap] + qu(:,1:prods);

% (ii) Use Judd's recursion to compute Chebyshev basis functions at each future state
r11 = ones(Chebygridpts*sims,1);
r21 = r11;
r12 = 2*(state1(:,1,1)-a1)/(b1-a1)-1;                          % transformed state variable 1 next period
r22 = 2*(state1(:,2,1)-a2)/(b2-a2)-1;                          % transformed state variable 2 next period
r13 = 2*r12.*r12 - r11;
r23 = 2*r22.*r22 - r21;
r14 = 2*r12.*r13 - r12;
r24 = 2*r22.*r23 - r22;
r15 = 2*r12.*r14 - r13;
r25 = 2*r22.*r24 - r23;
r16 = 2*r12.*r15 - r14;
r26 = 2*r22.*r25 - r24;

Xfuture1 =[ r11.*r21 r11.*r22 r11.*r23 r11.*r24];
Xfuture2 =[ r12.*r21 r12.*r22 r12.*r23 r12.*r24];
Xfuture3 =[ r13.*r21 r13.*r22 r13.*r23 r13.*r24];
Xfuture4 =[ r14.*r21 r14.*r22 r14.*r23 r14.*r24];

% EXPONENTIATED EXPECTED FUTURE VALUE OF WAITING
eVV1 = exp( Xfuture1*thetaChebyshev01(1:Chebyorder) + Xfuture2*thetaChebyshev01(Chebyorder+1:2*Chebyorder) + Xfuture3*thetaChebyshev01(2*Chebyorder+1:3*Chebyorder) + Xfuture4*thetaChebyshev01(3*Chebyorder+1:4*Chebyorder) );
eVV2 = exp( Xfuture1*thetaChebyshev02(1:Chebyorder) + Xfuture2*thetaChebyshev02(Chebyorder+1:2*Chebyorder) + Xfuture3*thetaChebyshev02(2*Chebyorder+1:3*Chebyorder) + Xfuture4*thetaChebyshev02(3*Chebyorder+1:4*Chebyorder) );

% SUM OF EXPONENTIATED FUTURE UTILITIES
expu11 = exp( Theta(1,1)+Theta(prods+1,1)*state1(:,1) + qu(:,prods+1) ) + exp( Theta(2,1) + Theta(prods+1,1)*state1(:,2) + qu(:,prods+2)) + eVV1;    % expected maximized future utility
expu12 = exp( Theta(1,2)+Theta(prods+1,2)*state1(:,1) + qu(:,prods+1) ) + exp( Theta(2,2) + Theta(prods+1,2)*state1(:,2) + qu(:,prods+2)) + eVV2;    % expected maximized future utility

% NPV OF EXPECTED MAXIMIZED FUTURE UTILITY
Y1 = delta*log(reshape(expu11,Chebygridpts,sims))*qwts;      % integrated next-period expected inclusive values using Gauss-Hermite quadrature
Y2 = delta*log(reshape(expu12,Chebygridpts,sims))*qwts;      % integrated next-period expected inclusive values using Gauss-Hermite quadrature

% UPDATED CHEBYSHEV WEIGHTS (this is analogous to the Galerkin method from Judd 1994)
thetaChebyshev11 = ProjChebpoly*Y1;                                 % update Chebyshev parameters using least squares
thetaChebyshev12 = ProjChebpoly*Y2;                                 % update Chebyshev parameters using least squares


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EVALUATE THE SHARES         %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
numer = exp(xi*ooR+Vjnetxi);            % this is the numerator (oo speeds-up expanding mean utility by number of types)
denom1 = sharesum*numer;
SS0 = 1./(1+denom1);                    % the outside share (or the inverse of the denominator)
Mt1 = M*(ones(T*Mkts,1)*mu(1));
Mt2 = M*(ones(T*Mkts,1)*mu(2));
for t=2:T,
    index = (0:Mkts-1)*T + t;
    Mt1(index,1) = SS0(index-1,1).*Mt1(index-1,1);    % mass of consumers at each mass point
    Mt2(index,1) = SS0(index-1,2).*Mt2(index-1,1);    % mass of consumers at each mass point
end
Mrt = [Mt1(:,1) Mt2(:,1)]./((Mt1(:,1)+Mt2(:,1))*ooR);       % time-specific mixing probabilities for each type "r"
denom = SS0(denomexpand,:);                                 % this expands the denominator
SS = numer.*denom;                                          % type-specific shares
lsh = log(sum(SS.*Mrt(denomexpand,:),2));                   % log of expected share (i.e. mean across types weighted by mixing probabilities)


% CONSTRAINTS -- Chebyshev coefficients must satisfy Bellman (note this could have been written in terms of the implied expected future value of
c = [thetaChebyshev01-thetaChebyshev11 ; thetaChebyshev02-thetaChebyshev12; lsh];