function LL = loglik_finmixNFP_price_2R(theta)

% LOGLIK_FINMIXNFP_PRICE_2R
% Log-Likelihood function for the Dynamic finite mixture random coefficients Logit.
% NFP algorithm used with nested calls to:
%   1) value function iteration
%   2) share inversion
%
% source: Dube, Fox and Su (2010)
% Code Revised: September 2010

global prods K Kp Kcov p X Xp Mkts T R quadpts_estim 
global SS Mrt
global Chebypolyobs xiindex psiindex

% SET UP PARAMETERS
Theta = reshape(theta([1:K 1:2 K+1]),K,R);            % preferences
muparam = [theta(K+1+1:K+1+(R-1))];                   % mixing parameters
mu = [muparam;1-sum(muparam)];
thetap = theta(K+1+(R-1)+1:K+1+(R-1)+Kp);             % pricing function parameters
thetacov = theta(K+1+(R-1)+Kp+1:K+1+(R-1)+Kp+Kcov);

cholCOVu = zeros(2*prods,2*prods)*mu(1);
cholCOVu(1,1) = thetacov(1);
cholCOVu(2,2) = thetacov(1);
cholCOVu(3,3) = thetacov(3);
cholCOVu(4,4) = thetacov(3);
cholCOVu(1,3) = thetacov(2);
cholCOVu(2,4) = thetacov(2);
COVu = cholCOVu(1:end,1:end)'*cholCOVu(1:end,1:end);
a = COVu(1,1); b = COVu(1,2); c = COVu(1,3); d = COVu(1,4); 
e = COVu(2,1); f = COVu(2,2); g = COVu(2,3); h = COVu(2,4); 
i = COVu(3,1); j = COVu(3,2); k = COVu(3,3); l = COVu(3,4); 
m = COVu(4,1); n = COVu(4,2); o = COVu(4,3); pp = COVu(4,4);
detCOVu = i*c*h*n-i*c*f*pp+a*f*k*pp-a*h*k*n;
invCOVu = [ [-(k/(-a*k+c^2))	, 0 , (c/(-a*k+c^2))	, 0]
            [0	, -(k/(-a*k+c^2))	, 0 , (c/(-a*k+c^2))]
            [(c/(-a*k+c^2))	, 0 , -(a/(-a*k+c^2))	, 0]
            [0	, (c/(-a*k+c^2))	, 0 , -(a/(-a*k+c^2))]];

qu = sqrt(2)*quadpts_estim*cholCOVu(1:end,1:end);    % quadrature points to integrate over the distribution of shocks

% EXPECTED VALUE OF WAITING AT OBSERVED STATES
thetaChebyshev = valuefunctions_Chebyshev2dim_estim(Theta,thetap,qu);      % Iterative solution of Bellman Equation via contraction-mapping
V0obs1 = Chebypolyobs*thetaChebyshev(:,1);
V0obs2 = Chebypolyobs*thetaChebyshev(:,2);
expVjnetxi = exp(X*Theta-repmat([V0obs1 V0obs2],prods,1));                 % exponentiated choice-specific value functions

% ITERATIVE SOLUTION OF DEMAND SHOCKS via contraction-mapping
xi = invertnfp_finmix(expVjnetxi,mu);

% PREDICTED PRICES AND IMPLIED SUPPLY SHOCKS
phat = Xp*thetap;
psi = p-phat;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EVALUATE THE LOG-LIKELIHOOD %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if max(max(isnan(xi))) == 1,
	LL = 1e+10+Theta(1);
else
    densityconstant = (2*pi)^(-2*prods/2)*(detCOVu)^(-.5);
    aa = invCOVu(1,1); bb = invCOVu(1,2); cc = invCOVu(1,3); dd = invCOVu(1,4);
    ee = invCOVu(2,1); ff = invCOVu(2,2); gg = invCOVu(2,3); hh = invCOVu(2,4); 
    ii = invCOVu(3,1); jj = invCOVu(3,2); kk = invCOVu(3,3); ll = invCOVu(3,4); 
    mm = invCOVu(4,1); nn = invCOVu(4,2); oo = invCOVu(4,3); ppp = invCOVu(4,4);
    phi = densityconstant*exp( -.5*(xi(xiindex(:,1)).*(cc*psi(psiindex(:,1))+kk*xi(xiindex(:,1)))+psi(psiindex(:,2)).*(ff*psi(psiindex(:,2))+nn*xi(xiindex(:,2)))+xi(xiindex(:,2)).*(hh*psi(psiindex(:,2))+ppp*xi(xiindex(:,2)))+psi(psiindex(:,1)).*(ii*xi(xiindex(:,1))+aa*psi(psiindex(:,1)))) );
    phi(phi(:,1)==0,1) = 1.e-300;
    logdetJacob = 0;
    density = zeros(T*Mkts,1);
    for tt=1:T*Mkts,
        index = (0:prods-1)*T*Mkts+tt;
        SSS = SS(index,1);
        Jacob = (diag(SSS)-SSS*SSS')*Mrt(tt,1);                   % Jacobian
        for rr = 2:R,
            SSS = SS(index,rr);
            Jacob = Jacob + (diag(SSS)-SSS*SSS')*Mrt(tt,rr);
        end
        logdetJacob = logdetJacob + log(1/det(Jacob));
    end
	LL = -sum(log(phi(:,1))) - logdetJacob;
end