function PROB = sharenfp_finmix(expxi,expVjnetxi,mu)

% IND_SHNORM
% This function computes the "individual" probabilities of choosing each
% brand for the the finite mixture random coefficients Logit.
%
% ARGUMENTS:
% expxi = vector of exponentiated demand shocks, \xi, sorted by product then market
% expVjnetxi = matrix of exponentiated type-specific utilities net of \xi
%
% OUTPUT:
% PROB = vector of expected market shares, sorted by product then market
%
% source: Dube, Fox and Su (2008)
% Code Revised: April 2008

global ooR sharesum denomexpand T R M Mkts
global SS PROB SS0 Mrt

numer = (expxi*ooR ).*expVjnetxi;       % this is the numerator (oo speeds-up expanding mean utility by number of types)
denom1 = sharesum*numer;
SS0 = 1./(1+denom1);                    % the outside share (or the inverse of the denominator)
Mt1 = M*(ones(T*Mkts,1)*mu(1));
Mt2 = M*(ones(T*Mkts,1)*mu(2));
for t=2:T,
    index = (0:Mkts-1)*T + t;
    Mt1(index,1) = SS0(index-1,1).*Mt1(index-1,1);    % mass of consumers at each mass point
    Mt2(index,1) = SS0(index-1,2).*Mt2(index-1,1);    % mass of consumers at each mass point
end
Mrt = [Mt1(:,1) Mt2(:,1)]./((Mt1(:,1)+Mt2(:,1))*ooR);       % time-specific mixing probabilities for each type "r"
denom = SS0(denomexpand,:);             % this expands the denominator
SS = numer.*denom;                      % type-specific shares
PROB = sum(SS.*Mrt(denomexpand,:),2);   % expected share (i.e. mean across types weighted by mixing probabilities)