function thetaChebyshev00 = valuefunctions_Chebyshev2dim(beta,sigma,qu)

% VALUEFUNCTIONS_Chebyshev2dim
% Compute expected value of waiting via NFP for the discrete random
% coefficients Logit demand model. Value Function Approximated
% parametrically using pre-determined number of Chebyshev polynomials on a
% pre-determined number of nodes.  Defined for a problem with a
% 2-dimensional state space (i.e. duopoly)
%
% Source: Dube, Fox and Su (2008)
% Revised: May 2008

global prods R delta qwts sims
global oo tolV
global thetaChebyshev0 Chebygrid Chebygridpts ooChebygrid Chebyorder a1 b1 a2 b2 oo1 oo2 ProjChebpoly

% (1) Pre-Compute the distribution of next-period prices and utilities at each state on the grid
p1_determ = reshape(kron(oo,[ooChebygrid Chebygrid(:,1:prods)])*sigma,Chebygridpts,prods);          % deterministic part of future prices given current software and prices
state1 = zeros(Chebygridpts*sims,2*prods);                                                          % next-period state
u1 = zeros(R,Chebygridpts,prods,sims);
for ss2 = 1:sims,
    index = (ss2-1)*Chebygridpts+1:ss2*Chebygridpts;
    st1 = [(p1_determ+ooChebygrid*qu(ss2,1:prods)) , ooChebygrid*qu(ss2,prods+1:2*prods)];          % pre-compute next-period distribution of consumer states (i.e. prices and \xi)
    state1(index,:) = st1;
    for rr = 1:R,
        u1(rr,:,:,ss2) = ooChebygrid*beta(1:prods,rr)'+beta(prods+1,rr)*st1(:,1:prods) + st1(:,prods+1:2*prods);  % next-period deterministic utilities from buying
    end
end
expu1 = squeeze(sum(exp(u1),3));            % sum the exponentiated utilities of each product (for each state)

% (2) Use Judd's recursion to compute Chebyshev basis functions at each future state
rhox1 = ones(Chebygridpts*sims,Chebyorder);
rhox2 = rhox1;
rhox1(:,2) = 2*(state1(:,1)-a1)/(b1-a1)-1;     % transformed state variable 1 next period
rhox2(:,2) = 2*(state1(:,2)-a2)/(b2-a2)-1;     % transformed state variable 2 next period
for ii=3:Chebyorder,
    rhox1(:,ii) = 2*rhox1(:,2).*rhox1(:,ii-1) - rhox1(:,ii-2);      %note used 2nd column of rho to obtain transformed state variable
    rhox2(:,ii) = 2*rhox2(:,2).*rhox2(:,ii-1) - rhox2(:,ii-2);
end
Xfuture = rhox1(:,oo1).*rhox2(:,oo2);       % tensor product of transformed state variables next period

thetaChebyshev00 = thetaChebyshev0;
iter = 0;
deltaV=1;

% ITERATE BELLMAN'S EQUATION UNTIL CONVERGENCE (CONTRACTION-MAPPING for NFP)
while deltaV>=tolV && iter<=1500,
    thetaChebyshev01 = thetaChebyshev0*0;
    if R==1,
        eVV = reshape(exp( Xfuture*thetaChebyshev00 ),Chebygridpts,sims);     % expected future exponentiated value of waiting
        Y = delta*log(eVV+expu1)*qwts;                              % integrated next-period expected inclusive values using Gauss-Hermite quadrature
        thetaChebyshev01 = ProjChebpoly*Y;                          % update Chebyshev parameters using least squares
    else
        for rr=1:R,
            eVV = reshape(exp( Xfuture*thetaChebyshev00(:,rr) ),Chebygridpts,sims);     % expected future exponentiated value of waiting
            Y = delta*log(eVV+squeeze(expu1(rr,:,:)))*qwts;         % integrated next-period expected inclusive values using Gauss-Hermite quadrature
            thetaChebyshev01(:,rr) = ProjChebpoly*Y;                % update Chebyshev parameters using least squares
        end
    end
    deltaV = max(max(max( abs(thetaChebyshev00-thetaChebyshev01) )));
    thetaChebyshev00 = thetaChebyshev01;
    iter = iter+1;
end

if sum(sum(isnan(thetaChebyshev00)))==0 && isreal(thetaChebyshev00)==1,
    thetaChebyshev0 = thetaChebyshev00;
end