function thetaChebyshev00 = valuefunctions_Chebyshev2dim_estim_noMAD(Theta,thetap,qu)

% VALUEFUNCTIONS_CHEBYSHEV2DIM_ESTIM_noMAD
% Compute expected value of waiting via NFP for the discrete random
% coefficients Logit demand model. Value Function Approximated
% parametrically using pre-determined number of Chebyshev polynomials on a
% pre-determined number of nodes.  Defined for a problem with a
% 2-dimensional state space (i.e. duopoly)
%
% Source: Dube, Fox and Su (2010)
% Revised: September 2010

global prods delta qwts sims
global tolV
global thetaChebyshev0 Chebygridpts Chebyorder Xstate1 Xstate2 a1 b1 a2 b2 ProjChebpoly

if nargin<4, oneiter=0; end;

% (i) Pre-Compute the distribution of next-period prices and utilities at each state on the grid
state1 = [Xstate1*thetap , Xstate2*thetap] + qu(:,1:prods);

% (ii) Use Judd's recursion to compute Chebyshev basis functions at each future state
r11 = ones(Chebygridpts*sims,1);
r21 = r11;
r12 = 2*(state1(:,1,1)-a1)/(b1-a1)-1;                          % transformed state variable 1 next period
r22 = 2*(state1(:,2,1)-a2)/(b2-a2)-1;                          % transformed state variable 2 next period
r13 = 2*r12.*r12 - r11;
r23 = 2*r22.*r22 - r21;
r14 = 2*r12.*r13 - r12;
r24 = 2*r22.*r23 - r22;
r15 = 2*r12.*r14 - r13;
r25 = 2*r22.*r24 - r23;
r16 = 2*r12.*r15 - r14;
r26 = 2*r22.*r25 - r24;

Xfuture1 =[ r11.*r21 r11.*r22 r11.*r23 r11.*r24];
Xfuture2 =[ r12.*r21 r12.*r22 r12.*r23 r12.*r24];
Xfuture3 =[ r13.*r21 r13.*r22 r13.*r23 r13.*r24];
Xfuture4 =[ r14.*r21 r14.*r22 r14.*r23 r14.*r24];

thetaChebyshev00 = thetaChebyshev0;
normV=1;
iter=0;
while normV>tolV && iter<=10000,
    % EXPONENTIATED EXPECTED FUTURE VALUE OF WAITING
    eVV1 = exp( Xfuture1*thetaChebyshev00(1:Chebyorder,1) + Xfuture2*thetaChebyshev00(Chebyorder+1:2*Chebyorder,1) + Xfuture3*thetaChebyshev00(2*Chebyorder+1:3*Chebyorder,1) + Xfuture4*thetaChebyshev00(3*Chebyorder+1:4*Chebyorder,1) );
    eVV2 = exp( Xfuture1*thetaChebyshev00(1:Chebyorder,2) + Xfuture2*thetaChebyshev00(Chebyorder+1:2*Chebyorder,2) + Xfuture3*thetaChebyshev00(2*Chebyorder+1:3*Chebyorder,2) + Xfuture4*thetaChebyshev00(3*Chebyorder+1:4*Chebyorder,2) );
    
    % SUM OF EXPONENTIATED FUTURE UTILITIES
    expu11 = exp( Theta(1,1)+Theta(prods+1,1)*state1(:,1) + qu(:,prods+1) ) + exp( Theta(2,1) + Theta(prods+1,1)*state1(:,2) + qu(:,prods+2)) + eVV1;    % expected maximized future utility
    expu12 = exp( Theta(1,2)+Theta(prods+1,2)*state1(:,1) + qu(:,prods+1) ) + exp( Theta(2,2) + Theta(prods+1,2)*state1(:,2) + qu(:,prods+2)) + eVV2;    % expected maximized future utility
    
    % NPV OF EXPECTED MAXIMIZED FUTURE UTILITY
    Y1 = delta*log(reshape(expu11,Chebygridpts,sims))*qwts;         % integrated next-period expected inclusive values using Gauss-Hermite quadrature
    Y2 = delta*log(reshape(expu12,Chebygridpts,sims))*qwts;         % integrated next-period expected inclusive values using Gauss-Hermite quadrature
 
    % UPDATED CHEBYSHEV WEIGHTS (this is analogous to the Galerkin method from Judd 1994)
    thetaChebyshev01(:,1) = ProjChebpoly*Y1;                        % update Chebyshev parameters using least squares
    thetaChebyshev01(:,2) = ProjChebpoly*Y2;                        % update Chebyshev parameters using least squares

    t = abs(thetaChebyshev00-thetaChebyshev01);
    normV = max(max(max(t)));
    thetaChebyshev00 = thetaChebyshev01;
    iter = iter+1;
end

%if sum(sum(isnan(thetaChebyshev00)))==0 && isreal(thetaChebyshev00)==1,
%    thetaChebyshev0 = thetaChebyshev00;
%end